/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/*
 * Interface checked by caps to determine the capability needed to
 * call methods on a component from potentially untrusted code.
 *
 * See also foo, which advertises whether untrusted code can get
 * services and create instances.
*/

#include "nsISupports.idl"

/**
 * Each method of this interface should return a string representing the
 * script capability needed to perform the operation on the target component.
 *
 * Return values of 'allAccess' or 'noAccess' unconditionally allow or deny
 * access to the operation.
 */
[scriptable, uuid(0dad9e8c-a12d-4dcb-9a6f-7d09839356e1)]
interface nsISecurityCheckedComponent : nsISupports {
  string canCreateWrapper(in nsIIDPtr iid);
  string canCallMethod(in nsIIDPtr iid, in wstring methodName);
  string canGetProperty(in nsIIDPtr iid, in wstring propertyName);
  string canSetProperty(in nsIIDPtr iid, in wstring propertyName);
};

// could put %{ block here containing macro'ed implementations of this
// interface for component developers' convenience.
