/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/slave/m-cen-lx-xr-ntly-0000000000000/build/dom/interfaces/html/nsIDOMHTMLUnknownElement.idl
 */

#ifndef __gen_nsIDOMHTMLUnknownElement_h__
#define __gen_nsIDOMHTMLUnknownElement_h__


#ifndef __gen_nsIDOMHTMLElement_h__
#include "nsIDOMHTMLElement.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif

/* starting interface:    nsIDOMHTMLUnknownElement */
#define NS_IDOMHTMLUNKNOWNELEMENT_IID_STR "aa044a2d-4e9b-4fc5-8ad2-666da9062b36"

#define NS_IDOMHTMLUNKNOWNELEMENT_IID \
  {0xaa044a2d, 0x4e9b, 0x4fc5, \
    { 0x8a, 0xd2, 0x66, 0x6d, 0xa9, 0x06, 0x2b, 0x36 }}

class NS_NO_VTABLE nsIDOMHTMLUnknownElement : public nsIDOMHTMLElement {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IDOMHTMLUNKNOWNELEMENT_IID)

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIDOMHTMLUnknownElement, NS_IDOMHTMLUNKNOWNELEMENT_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIDOMHTMLUNKNOWNELEMENT \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIDOMHTMLUNKNOWNELEMENT(_to) \
  /* no methods! */

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIDOMHTMLUNKNOWNELEMENT(_to) \
  /* no methods! */

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsDOMHTMLUnknownElement : public nsIDOMHTMLUnknownElement
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIDOMHTMLUNKNOWNELEMENT

  nsDOMHTMLUnknownElement();

private:
  ~nsDOMHTMLUnknownElement();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsDOMHTMLUnknownElement, nsIDOMHTMLUnknownElement)

nsDOMHTMLUnknownElement::nsDOMHTMLUnknownElement()
{
  /* member initializers and constructor code */
}

nsDOMHTMLUnknownElement::~nsDOMHTMLUnknownElement()
{
  /* destructor code */
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIDOMHTMLUnknownElement_h__ */
