/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

[scriptable, builtinclass, uuid(86c7c396-9c2b-4d45-9c7c-b0dcda024105)]
interface nsIDOMDeviceProximityEvent : nsIDOMEvent
{
  [noscript] void initDeviceProximityEvent(in DOMString eventTypeArg,
                                           in boolean canBubbleArg,
                                           in boolean cancelableArg,
                                           in double value,
                                           in double min,
                                           in double max);
  readonly attribute double value;
  readonly attribute double min;
  readonly attribute double max;
};


dictionary DeviceProximityEventInit : EventInit
{
   double value = Infinity;
   double min = -Infinity;
   double max = Infinity;
};
